@@ -1,8 +1,11 @@
-# Copyright © The University of Edinburgh, 2022.
+# Copyright © The University of Edinburgh, 2023.
 # Development has been supported by GSK.
 
 from .phenonaut import Phenonaut
 import phenonaut.metrics
+import phenonaut.metrics.performance
+import phenonaut.metrics.utils
+
 import phenonaut.output
 from phenonaut.data import PlatemapQuerier
 import phenonaut.packaged_datasets
