@@ -2,9 +2,9 @@
 # Development has been supported by GSK.
 
 from .tcga import TCGA
-from .cmap import CMAP
+from .cmap import CMAP, CMAP_Level4
 from .iris import Iris
 from .iris import Iris_2_views
 from .breast_cancer import BreastCancer
 
-
+from .metadata_moa import MetadataJUMPMOACompounds, MetadataBROADLincsCellPaintingMOAs
