@@ -4,6 +4,7 @@
 import warnings
 from random import random
 from pandas.errors import DataError
+from phenonaut import data
 from phenonaut.data import Dataset
 from typing import Callable, Iterable, List, Optional, Union
 from pathlib import Path
@@ -103,7 +104,6 @@ class Phenonaut:
         dataframe_name: Optional[Union[str, list[str]]] = None,
         init_hash: Optional[Union[str, bytes]] = None,
     ):
-
         if init_hash is None:
             self.init_hash = b""
         else:
@@ -144,11 +144,19 @@ class Phenonaut:
             # Append to datasets, setting features and name.
             if dataframe_name is None:
                 self.datasets.append(
-                    Dataset(dataset_name="sklearnBunch", input_file_path=df, metadata={"features": dataset.feature_names})
+                    Dataset(
+                        dataset_name="sklearnBunch",
+                        input_file_path=df,
+                        metadata={"features": dataset.feature_names},
+                    )
                 )
             else:
                 self.datasets.append(
-                    Dataset(dataset_name=dataframe_name, input_file_path=df, metadata={"features": dataset.feature_names})
+                    Dataset(
+                        dataset_name=dataframe_name,
+                        input_file_path=df,
+                        metadata={"features": dataset.feature_names},
+                    )
                 )
 
             if name is not None:
@@ -161,13 +169,14 @@ class Phenonaut:
                 dataset = Path(dataset)
             if dataframe_name is None:
                 dataframe_name = str(dataset)
-            dataset = Dataset(dataset_name=dataframe_name, input_file_path=dataset, metadata=metadata, kind=kind)
+            dataset = Dataset(
+                dataset_name=dataframe_name, input_file_path=dataset, metadata=metadata, kind=kind
+            )
 
         # If its a Dataset, or list of datasets, directly add it to self.datasets.
         if isinstance(dataset, (Dataset, pd.DataFrame)):
             dataset = [dataset]
         if isinstance(dataset, list):
-
             ds_types = list(set(map(type, dataset)))
 
             if len(ds_types) > 1:
@@ -201,7 +210,9 @@ class Phenonaut:
                         f"dataset_metadata contained {len(metadata)} metadata dictionaries, but dataset contained {len(dataset)} pd.DataFrames"
                     )
                 for ds_name, df, ds_metadata in zip(dataframe_name, dataset, metadata):
-                    self.datasets.append(Dataset(dataset_name=ds_name, input_file_path=df, metadata=ds_metadata))
+                    self.datasets.append(
+                        Dataset(dataset_name=ds_name, input_file_path=df, metadata=ds_metadata)
+                    )
                 return
 
             if name is not None:
@@ -254,9 +265,7 @@ class Phenonaut:
         """
         if isinstance(dataset, str):
             if dataset not in self.keys():
-                raise KeyError(
-                    f"'{dataset}' not found as a name in datasets ({self.keys()})"
-                )
+                raise KeyError(f"'{dataset}' not found as a name in datasets ({self.keys()})")
             return self.datasets[self.keys().index(dataset)]
         elif isinstance(dataset, int):
             return self.datasets[dataset]
@@ -267,7 +276,7 @@ class Phenonaut:
                 f"Attempting to get dataset, but {type(dataset)} was supplied. It should be str name, int giving the dataset location in the Phenonaut.datasets list, or a list/tuple of names/indexes to return multiple datasets"
             )
 
-    def __setitem__(self, name: str, dataset: Dataset) -> None:
+    def __setitem__(self, name: Union[str, int], dataset: Dataset) -> None:
         """Allows asignment of Datasets to Phenonaut object by name
 
         Parameters
@@ -277,8 +286,15 @@ class Phenonaut:
         dataset : Dataset
             New Dataset to be stored in Phenonaut object
         """
-        dataset.name = name
-        self.datasets.append(dataset)
+        if isinstance(name, int):
+            self.datasets[name] = dataset
+        else:
+            dataset.name = name
+            if name in self.keys():
+                name = self.get_dataset_index_from_name(name)
+                self.datasets[name] = dataset
+            else:
+                self.datasets.append(dataset)
 
     def __delitem__(self, key: Union[str, int]):
         """Allows the deletion of Datasets stored in a Phenonaut object
@@ -304,9 +320,7 @@ class Phenonaut:
         """
         if isinstance(key, str):
             if key not in self.keys():
-                raise KeyError(
-                    f"'{key}' not found as a name in datasets ({self.keys()})"
-                )
+                raise KeyError(f"'{key}' not found as a name in datasets ({self.keys()})")
             del self.datasets[self.keys().index(key)]
             return
         elif isinstance(key, int):
@@ -419,7 +433,7 @@ class Phenonaut:
         input_file_path: Union[Path, str],
         metadata: dict = None,
         h5_key: Optional[str] = None,
-        features:Optional[list[str]]=None,
+        features: Optional[list[str]] = None,
     ):
         """Load a dataset from a CSV, optionally suppying metadata and a name
 
@@ -442,11 +456,18 @@ class Phenonaut:
         """
         if dataset_name is None and metadata is not None:
             dataset_name = metadata.pop("dataset_name", f"Dataset {len(self.datasets)+1}")
-        if features !=None:
-            metadata['features']=features
-        self.datasets.append(Dataset(dataset_name=dataset_name, input_file_path=input_file_path, metadata=metadata))
+        if features != None:
+            metadata["features"] = features
+        self.datasets.append(
+            Dataset(dataset_name=dataset_name, input_file_path=input_file_path, metadata=metadata)
+        )
 
-    def combine_datasets(self, dataset_ids_to_combine:Optional[Union[list[str], list[int]]]=None, new_name:Optional[str]=None, features: list = None):
+    def combine_datasets(
+        self,
+        dataset_ids_to_combine: Optional[Union[list[str], list[int]]] = None,
+        new_name: Optional[str] = None,
+        features: list = None,
+    ):
         """Combine multiple datasets into a single dataset
 
         Often, large datasets are split across multiple CSV files. For example,
@@ -478,8 +499,8 @@ class Phenonaut:
         if dataset_ids_to_combine == []:
             raise ValueError("Empty list supplied as indexes of datasets to combine")
         if dataset_ids_to_combine is None:
-            dataset_ids_to_combine=[idx for idx in range(len(self.datasets))]
-        
+            dataset_ids_to_combine = [idx for idx in range(len(self.datasets))]
+
         dataframes_to_combine = [self[i].df for i in dataset_ids_to_combine]
         datasets_to_combine = [self[i] for i in dataset_ids_to_combine]
         new_metadata = {}
@@ -497,9 +518,7 @@ class Phenonaut:
                     )
         if new_name is None:
             new_name = f"Combined_dataset from datasets[{dataset_ids_to_combine}]"
-        self.datasets.append(
-            Dataset(dataset_name=new_name,input_file_path=None, metadata=None)
-        )
+        self.datasets.append(Dataset(dataset_name=new_name, input_file_path=None, metadata=None))
         self.datasets[-1].df = new_df
         self.datasets[-1]._metadata = new_metadata
         self.datasets[-1].features = (
@@ -783,7 +802,6 @@ class Phenonaut:
             for c in itertools_combinations(ds_names, i)
         )
 
-
     def aggregate_dataset(
         self,
         composite_identifier_columns: list[str],
@@ -874,16 +892,13 @@ class Phenonaut:
             By default "mean".
 
         """
-        
+
         if isinstance(datasets, (int, str)):
             datasets = [datasets]
-        datasets=list(datasets) # In case generator is consumed by new_names_or_prefix below.
+        datasets = list(datasets)  # In case generator is consumed by new_names_or_prefix below.
 
         if isinstance(new_names_or_prefix, str):
-            new_names_or_prefix = [
-                f"{new_names_or_prefix}{self[idx].name}"
-                for idx in datasets
-            ]
+            new_names_or_prefix = [f"{new_names_or_prefix}{self[idx].name}" for idx in datasets]
 
         if len(new_names_or_prefix) != len(datasets):
             raise ValueError(
@@ -946,8 +961,8 @@ class Phenonaut:
             Datasets contain common feature columns, cannot merge.
         ValueError
             No common columns found to perform merge.
-            
-        """        
+
+        """
         if ids is None:
             ids = self.get_dataset_index_from_name(self.keys())
             if len(ids) != 2:
@@ -959,7 +974,9 @@ class Phenonaut:
                 f"merge_datasets merges 2 Datasets at a time. The ids argument contained {len(ids)} dataset indexes"
             )
         if any(f1 in self[ids[1]].features for f1 in self[ids[0]].features):
-            raise ValueError(f"Datasets shared common features, cannot merge, common features were {set(self[ids[0]].features).intersection(set(self[ids[1]].features))}")
+            raise ValueError(
+                f"Datasets shared common features, cannot merge, common features were {set(self[ids[0]].features).intersection(set(self[ids[1]].features))}"
+            )
         if common_merge_columns is None:
             common_merge_columns = list(
                 set.intersection(*map(set, [self[id].df.columns.tolist() for id in ids]))
@@ -968,18 +985,22 @@ class Phenonaut:
             raise ValueError("No common columns found in dataframes to perform merge")
         new_df = pd.merge(self[ids[0]].df, self[ids[1]].df, on=common_merge_columns)
         # Uniquify features, preserving order
-        merged_features=self[ids[0]].features+self[ids[1]].features
+        merged_features = self[ids[0]].features + self[ids[1]].features
 
-        self.datasets.append(Dataset(dataset_name=new_dataset_name,input_file_path=new_df, metadata={'features':merged_features}))
+        self.datasets.append(
+            Dataset(
+                dataset_name=new_dataset_name,
+                input_file_path=new_df,
+                metadata={"features": merged_features},
+            )
+        )
 
     def save(self, output_filename: Union[str, Path], overwrite_existing: bool = False) -> None:
         """Save Phenonaut object and contained Data to a pickle
 
         Writes a gzipped Python pickle file. If no compression, or another
         compressison format is required, then the user should use a custom
-        pickle.dump and not rely on this helper function.  Warning, SHA256
-        '_hashlib.HASH' objects are not serialisable, so crytographic hashes
-        and proof of manipulations are lost upon saving.
+        pickle.dump and not rely on this helper function.
 
         Parameters
         ----------
@@ -991,8 +1012,39 @@ class Phenonaut:
         for ds in self.datasets:
             ds.sha256 = b64encode(ds.sha256.digest()).decode("utf-8")
         output_filename = check_path(output_filename)
+        if output_filename.exists() and not overwrite_existing:
+            raise FileExistsError(f"{output_filename} exists, and overwrite_existing was False")
+        self.last_saved_pickle_filename = output_filename
         with gzip.open(output_filename, "wb") as f:
             pickle.dump(self, f)
+        for ds in self.datasets:
+            ds.sha256 = sha256(ds.sha256.encode("utf-8"))
+
+    def revert(self) -> None:
+        """Revert a Phenonaut object that was recently saved to it's previous state
+
+        Upon calling save on a Phenonaut object, the record stores the output file location,
+        allowing a quick was to revert changes made to a file by calling .revert(). This
+        returns the object to it's saved state.
+
+        Raises
+        ------
+        FileNotFoundError
+            File not found, Phenonaut object has never been written out.
+
+        """
+        if hasattr(self, "last_saved_pickle_filename"):
+            if not self.last_saved_pickle_filename.exists():
+                raise FileNotFoundError(f"Could not find file {self.last_saved_pickle_filename}")
+            with gzip.open(self.last_saved_pickle_filename, "rb") as f:
+                phe = pickle.load(f)
+                for ds in phe.datasets:
+                    ds.sha256 = sha256(ds.sha256.encode("utf-8"))
+                self.__dict__ = phe.__dict__
+        else:
+            raise FileNotFoundError(
+                "Called revert on a Phenonaut object which has never been saved.  Save it first, before calling revert."
+            )
 
     @classmethod
     def load(cls, filepath: Union[str, Path]) -> "Phenonaut":
@@ -1023,3 +1075,38 @@ class Phenonaut:
             for ds in phe.datasets:
                 ds.sha256 = sha256(ds.sha256.encode("utf-8"))
             return phe
+
+    def get_df_features_perturbation_column(
+        self, ds_index=-1, quiet: bool = False
+    ) -> tuple[pd.DataFrame, list[str], Union[str, None]]:
+        """Helper function to obtain DataFrame, features and perturbation column name.
+
+        Some Phenonaut functions allow passing of a Phenonaut object, or DataSet. They
+        then access the underlying pd.DataFrame for calculations. This helper function
+        is present on Phenonaut objects and Dataset objects, allowing more concise
+        code and less replication when obtaining the underlying data. If multiple
+        Datasets are present, then the last added Dataset is used to obtain data, but
+        this behaviour can be changed by passing the ds_index argument.
+
+        Parameters
+        ----------
+        ds_index : int, optional
+            Index of the Dataset to be returned. By default -1, which uses the
+            last added Dataset.
+        quiet : bool
+            When checking if perturbation is set, check without inducing a
+            warning if it is None.
+
+        Returns
+        -------
+        tuple[pd.DataFrame, list[str], str]
+            Tuple containing the Dataframe, a list of features and the
+            perturbation column name.
+        """
+        return (
+            self[ds_index].df,
+            self[ds_index].features,
+            self[ds_index]._metadata.get("perturbation_column", None)
+            if quiet
+            else self[ds_index].perturbation_column,
+        )
